use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AnnoCPAN
CPAN
Github
HTTPS
IO
IPC
JSON
Koivunalho
Mikko
POSIX
Params
Readonly
Rezić
RunExternal
STDERR
STDOUT
Slaven
Storable
Subdirectory
TODO
Timestamp
alnum
ascii
aws
backticks
cas
committer
committers
dir
filename
http
https
io
json
lib
licensable
lv
mailmap
mikkoi
msg
multi
params
runtime
scalability
semafores
slaven
startup
subitem
subparam
succcessful
successfull
timestamp
tradename
unreserves
url
webhooks
wildcard
wildcards
